%
% Copyright (c) 2013, Weicong Ding, Mohammad H. Rohban, Prakash Ishwar, 
% and Venkatesh Saligrama. All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
% 1) Redistributions of source code must retain the above copyright
%    notice, this list of conditions and the following disclaimer.
%
% 2) Redistributions in binary form must reproduce the above copyright
%    notice, this list of conditions and the following disclaimer in the
%    documentation and/or other materials provided with the distribution.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
% A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
% HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
% LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
% DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
% THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%

% This script is used for generate a synthetic document collection based on
% Latent Dirichelet Allocation (Blei et al., 2003) with a separable topic
% matrix \beta. 
% Used for synthetic experiment in Ding et al, 2013.( Topic Discovery through 
% Data Dependent and Random Projection ICML'13)

% Sepecify the meta model parameters 
k = 5;  % # topics
M = 500; % # documents 
Nmeans = 20; % # words in each documents
rou = 0.2; % ratio of beta 1 parts
B2 = 1; % the rate B2 is the hyperparameter of the rest of the topic matrix
        % setting B2 = 1 to make it uniform distributed

% parameter of the TOPIC-DOCUMENT Matrix \theta
a0 = 0.1; % alphas of the Dirichelet prior 
alpha = ones(k,1)*a0;
theta = sample_dirichlet(alpha, M)'; 

% Generate Seperable TOPIC-WORD matrix
rw = (1-rou)/rou;
U = ceil(rou*W) ;
beta1 = genseperabletopic(U,k);
beta2=sample_dirichlet(ones(W-U,1)*B2,k)';
beta = [beta1; rw*beta2];
beta = normalizeTopic(beta);

% Check if a topic is separable 
[flag,anchor] = checkseperable(beta, k);
[beta, label] = reorder(beta,anchor);
U2 =length(label);
label = [label,(k+1)*ones(1,W-U2)];
clear alpha  a0 rou rw B2 beta1 beta2  anchor

% Generate WORD-DOCUMENT Matrix
docdist = beta * theta;
doccnt = sparse(zeros(size(docdist)));
% Sample the Synthetic Bag-Of-Words
Ndoc=zeros(M,1);
for i=1:M
    N=random('poiss', Nmeans, 1,1);
    Ndoc(i)=N;
    P=docdist(:,i);
    doccnt(:,i)=mnrnd(N,P);
    dochist(:,i)=doccnt(:,i)/N;
end
Nw = sum(doccnt,2);
clear Ndoc N i P

% docdist - the distribution methods for document collection. The A matrix
% as in Ding et al., 2013
% doccnt - term by document matrix, stores words counts
% dochist - the empirical document histogram. The X matrix as in Ding et
% al., 2013
