%
% Copyright (c) 2013, Weicong Ding, Mohammad H. Rohban, Prakash Ishwar, 
% and Venkatesh Saligrama. All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
% 1) Redistributions of source code must retain the above copyright
%    notice, this list of conditions and the following disclaimer.
%
% 2) Redistributions in binary form must reproduce the above copyright
%    notice, this list of conditions and the following disclaimer in the
%    documentation and/or other materials provided with the distribution.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
% A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
% HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
% LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
% DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
% THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%

% This is a demo script to use algorithm in Ding et al., 2013 to discover
% the topics from text dataset. 
% This demo uses the NIPS article dataset which can be obtained from http://archive.ics.uci.edu/ml/datasets/Bag+of+Words
    
% Load data and remove some rare words beyond a certain threshold
load nips_demo.mat
% a standard English stop word list has to be removed
k = 40; % set the number of topics
[W,M]=size(counts);
clear aw_counts authors_names docs_names docs_authors
Nw=sum(counts,2);
thre = 100;
trustword = find(Nw>thre);
testmtx=counts(trustword,:);
Wtrust=length(trustword);
% Remove words below a certain word frequency


% random projection
testmtx = normalizeTopic(testmtx')'; % row normalization 
p=5000; % specify the # of random projection iid spherical Gaussian
score=zeros(Wtrust,1);
distancetemp=zeros(Wtrust,1);
el1=1.0;
for i=1:p
    display(['start the project' ]);
    i
    P = randn(M,1);
    P = P/norm(P);
        cost=testmtx*P;
    [Y,Imax] = max(cost);
    [Y,Imin] = min(cost);
    Selected =union(Imax,Imin);
    for kk=1:length(Selected)
        wordid = Selected(kk);
        tempmtx = sparse(ones(Wtrust,1))*testmtx(wordid,:);
        distancetemp = sum(abs(tempmtx-testmtx),2);
        % find the l1-neighors
        idxi = find(distancetemp<el1);
        score(idxi) =  score(idxi) + 1;
    end
    uselected = sum(score>0)
end
% find high-freq. words being maximum 
thr=1;
idxcand = find(score>=thr);
Ngood = length(idxcand);
u=trustword(idxcand);

% clustering candidate novel words
candidateandoutlier=testmtx(idxcand,:);
CMat = SparseCoefRecovery(candidateandoutlier',1,'Lasso',10);
CKSym = BuildAdjacency(CMat,100);
[Grps ] = SpectralClustering(CKSym,k);
[group3,IDX]=sort(Grps(:,3));
usort = trustword(idxcand(IDX));
% store the novel words
tw= cell(k,50);
for i=1:k
    a=find(Grps(:,3)==i);
    for j=1:length(a)
        tw{i,j}=words{u(a(j))};
    end
end
% topic estimation 
unclassedwordlabel=setdiff([1:W],usort);
groupdata=counts(usort,:);
ungroupdata=counts(unclassedwordlabel,:);

groupdata = normalizeTopic(groupdata')';
ungroupdata = normalizeTopic(ungroupdata')';

cvxmtx = zeros(k,length(usort));
for ii=1:k
    cvxmtx(ii,:)= group3==ii;
end

% lamba =1 , norm =1  reg = Inf
% lamba =0.1 , norm =2  reg = Inf
lambda=0.1;

for ic=1:length(unclassedwordlabel)
    xi = ungroupdata(ic,:);
            cvx_begin;
            cvx_precision high
            variable c(1,Ngood);
            minimize( norm(c * groupdata -xi ,2)+ lambda*sum(norms(cvxmtx*diag(c),Inf,2)));
     %       minimize( norm(c * groupdata -xi, 1)+ lambda* sum(norms(cvxmtx*c',1,2)) );
            subject to
             c >= 0;
%            c * groupdata == xi;
    cvx_end;
    BMat(ic,:)=c;
end
% renoamlized the topic matrix 
betamtx = cvxmtx * BMat';
betamtx=betamtx';
beta(unclassedwordlabel,:)=betamtx;
beta(usort,:)=cvxmtx';
for i=1:W
    beta(i,:)=Nw(i)*beta(i,:);
end

% out put high-frequency words for each topics 
for kk=1:k
    wa =beta(:,kk);
    [wa,IDX]=sort(wa,'descend');
    for j=1:20
        tpw{kk,j}=words{IDX(j)};
    end
end

%Notes: tpw stores the high-frequency words for each topic.
% beta stores the estimated topic matrix